#ifndef _VECTOR_H
#define _VECTOR_H

struct vector_t {
  int nbElem;
  char **elem;
};

/**
   Renvoie un pointeur vers un nouveau vecteur vide ou bien NULL s'il y a eu
   un probleme.
 */
struct vector_t *newVec();

/**
   Libere toutes les structures de donnees accessible a partir de pv.
   Libere egalement la zone pointee par pv
 */
void deleteVec(struct vector_t *pv);

/**
   Ajoute une copie de la string s a la fin du vecteur pv. Renvoie le
   rang dans le vecteur de la chaine ajoutee ou bien -1 s'il y a eu un
   problème.
 */
int pushBack(struct vector_t *pv, char *s);

/**
   Renvoie l'element place a la case rank dans pv ou NULL si rank est en
   dehors des bornes de pv.
 */
char *getElem(const struct vector_t *pv, int rank);

/**
   Cree un nouveau vecteur en initialisant son contenu avec les informations
   trouvees dans le fichier de nom filename.
   S'il y a un souci dans l'acces au fichier filename, affiche un message
   d'erreur et renvoie NULL.
   NB : le fichier est structure de la maniere suivante
   - Les 4 premiers octets contiennent le nombre d'elements du vecteur, code
     en binaire
   - Ensuite, pour chaque element du vecteur, on trouve 4 octets contenant
     la longueur de la chaine de caracteres de l'elemnts, puis les octets des
     differents caracteres de la chaine (sans le '\0' final).
 */
struct vector_t *loadVec(char *filename);

/**
   Sauvegarde le vecteur pv dans le fichier binaire de nom filename (format du
   fichier explique dans loadVec).
   Renvoie 0 si tout s'est bien passe et -1 sinon (apres avoir affiche un
   message d'erreur explicatif)
 */
int saveVec(struct vector_t *pv, char *filename);

#endif /* _VECTOR_H */
