#include <stdarg.h>
#include <stddef.h>
#include <setjmp.h>
#include "cmocka.h"
#include "vector.h"

static void test_newVec_pushBack_getElem(void **state) {
  struct vector_t *pv = newVec();
  assert_non_null(pv);

  assert_int_equal(0, pushBack(pv, "abc"));
  assert_int_equal(1, pushBack(pv, "ab"));
  assert_int_equal(2, pushBack(pv, "a"));

  char *s = getElem(pv, -1);  
  assert_null(s);  

  s = getElem(pv, 0);
  assert_non_null(s);  
  assert_string_equal("abc", s);
		   
  s = getElem(pv, 1);
  assert_non_null(s);  
  assert_string_equal("ab", s);
		   
  s = getElem(pv, 2);
  assert_non_null(s);  
  assert_string_equal("a", s);

  s = getElem(pv, 3);  
  assert_null(s);  
 
  deleteVec(pv);
  pv = NULL;
}

static void test_loadVec(void **state) {
  struct vector_t *pv = loadVec("pourTestUnitaireLoadVec.dat");
  assert_non_null(pv);

  char *s = getElem(pv, 0);
  assert_non_null(s);  
  assert_string_equal("abc", s);
		   
  s = getElem(pv, 1);
  assert_non_null(s);  
  assert_string_equal("ab", s);
		   
  s = getElem(pv, 2);
  assert_non_null(s);  
  assert_string_equal("a", s);

  s = getElem(pv, 3);  
  assert_null(s);  

  deleteVec(pv);
  pv = NULL;
}

static void test_saveVec(void **state) {
  struct vector_t *pv = newVec();  
  assert_non_null(pv);

  assert_int_equal(0, pushBack(pv, "abc"));
  assert_int_equal(1, pushBack(pv, "ab"));
  assert_int_equal(2, pushBack(pv, "a"));

  assert_int_equal(0, saveVec(pv, "pourTestUnitaireSaveVec.dat"));

  deleteVec(pv);

  pv = loadVec("pourTestUnitaireSaveVec.dat");
  assert_non_null(pv);

  char *s = getElem(pv, 0);
  assert_non_null(s);  
  assert_string_equal("abc", s);
		   
  s = getElem(pv, 1);
  assert_non_null(s);  
  assert_string_equal("ab", s);
		   
  s = getElem(pv, 2);
  assert_non_null(s);  
  assert_string_equal("a", s);

  s = getElem(pv, 3);  
  assert_null(s);  

  deleteVec(pv);
  pv = NULL;
}

int main(void) {
  const struct CMUnitTest tests[] = {
    cmocka_unit_test(test_newVec_pushBack_getElem),
    cmocka_unit_test(test_loadVec),
    cmocka_unit_test(test_saveVec),
  };
  return cmocka_run_group_tests(tests, NULL, NULL);
}
